package apirtdatabase

import (
	"errors"
	"fmt"

	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"git.sr.ht/~mil/mobsql"
	"github.com/go-playground/validator/v10"
)

func RTDatabase(runtime *apirtinit.MobrouteRuntime, params *DatabaseParams) (*DatabaseResponse, error) {
	utillog.Infoj("Running RTDatabase against input", params)

	if err := validator.New(validator.WithRequiredStructEnabled()).Struct(params); err != nil {
		return nil, fmt.Errorf("Validation failed request: %v", err)
	} else if response, err := runMobsqlFn(
		runtime.MobsqlRuntime,
		params.Op,
		params.FeedIDs,
		params.LoadCustomGTFSURI,
	); err != nil {
		return nil, err
	} else {
		return response, nil
	}
}

func runMobsqlFn(mobsqlRuntime *mobsql.MobsqlRuntime, op string, feedIDs []int, loadCustomGTFSURI *string) (*mobsql.FeedOpResult, error) {
	if op == "load" {
		return mobsql.FeedLoad(mobsqlRuntime, feedIDs)
	} else if op == "loadcustomgtfs" {
		if loadCustomGTFSURI == nil {
			return nil, errors.New("Must specify loadcustomgtfs_uri when using loadcustomgtfs op")
		}
		return mobsql.FeedLoadCustomGTFS(mobsqlRuntime, feedIDs[0], *loadCustomGTFSURI)
	} else if op == "compute" {
		return mobsql.FeedCompute(mobsqlRuntime, feedIDs)
	} else if op == "purge_gtfs" {
		return mobsql.FeedPurge(mobsqlRuntime, feedIDs, mobsql.PurgeTablesOptionGTFS)
	} else if op == "purge_computed" {
		return mobsql.FeedPurge(mobsqlRuntime, feedIDs, mobsql.PurgeTablesOptionComputed)
	} else if op == "purge_all" {
		return mobsql.FeedPurge(mobsqlRuntime, feedIDs, mobsql.PurgeTablesOptionAll)
	} else {
		return nil, errors.New("Invalid op")
	}
}
