package apirtroute

import (
	"encoding/json"
	"git.sr.ht/~mil/mobroute/api/apirtroute/diagnosticst"
	"git.sr.ht/~mil/mobroute/api/apirtroute/formatterdiagnostics"
	"git.sr.ht/~mil/mobroute/api/apirtroute/formattergeojson"
	"git.sr.ht/~mil/mobroute/api/apirtroute/formatterlegs"
	"git.sr.ht/~mil/mobroute/api/apirtroute/formattermapurl"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
)

func formatRouteResponse(
	diagnostics *diagnosticst.RouteDiagnostics,
	connectionsVerbose *[]dbt.ConnectionVerbose,
	transfers *[]dbt.Transfer,
	routeParams *RouteParams,
) *RouteResponse {
	var (
		r                 = RouteResponse{}
		formatConnections = utilfuncs.ItemInArray("connections", routeParams.OutputFormats)
		formatLegs        = utilfuncs.ItemInArray("legs", routeParams.OutputFormats)
		formatGeojson     = utilfuncs.ItemInArray("geojson", routeParams.OutputFormats)
		formatMapurl      = utilfuncs.ItemInArray("mapurl", routeParams.OutputFormats)
		formatDiagnostics = utilfuncs.ItemInArray("diagnostics", routeParams.OutputFormats)
		formatRequest     = utilfuncs.ItemInArray("request", routeParams.OutputFormats)
	)

	// Formatters which depend / are derived from the base 'legs' format
	if formatLegs || formatGeojson || formatMapurl || formatDiagnostics {
		routeLegs := formatterlegs.FormatLegs(
			connectionsVerbose,
			transfers,
			routeParams.Time,
			routeParams.WalkspeedKmHr,
			routeParams.From,
			routeParams.To,
		)
		if formatLegs {
			r.RouteLegs = routeLegs
		}
		if formatGeojson {
			geojsonMetadata, _ := json.Marshal(routeParams)
			r.GeoJSON = formattergeojson.FormatGeoJSON(routeLegs, string(geojsonMetadata))
		}
		if formatMapurl {
			geojsonMetadata, _ := json.Marshal(routeParams)
			r.MapURL = ptr(formattermapurl.FormatMapURL(formattergeojson.FormatGeoJSON(routeLegs, string(geojsonMetadata))))
		}
		if formatDiagnostics {
			r.RouteDiagnostics = formatterdiagnostics.FormatDiagnosticsExtra(diagnostics, routeLegs)
		}
	}

	// Raw connections dump
	if formatConnections {
		r.Connections = connectionsVerbose
	}

	// Raw request dump
	if formatRequest {
		r.Request = routeParams
	}

	return &r
}
