package formatterlegs

import (
	"fmt"
	"time"

	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utillog"
)

func FormatConnectionsToLegTransfer(
	transfers *[]dbt.Transfer,
	legTransferFrom, legTransferTo *RouteLeg,
) *RouteLeg {
	var useTransfer *dbt.Transfer

	if useTransfer = transferLookup(
		transfers,
		*legTransferFrom.TripToSIDU,
		*legTransferTo.TripFromSIDU,
	); useTransfer == nil {
		utillog.Fatalf(
			"Should be impossible - failed to lookup transfer for (%s -> %s) in formatter?",
			*legTransferFrom.TripToSIDU,
			*legTransferTo.TripFromSIDU,
		)
	}

	var (
		transferDurationTotal     time.Duration = legTransferTo.LegBeginTime.Sub(legTransferFrom.LegEndTime)
		transferDurationActive, _               = time.ParseDuration(fmt.Sprintf("%ds", useTransfer.TransferSeconds))
		transferDurationDwell     time.Duration = transferDurationTotal - transferDurationActive
	)
	return &RouteLeg{
		LegType:          "transfer",
		LegDuration:      transferDurationTotal.String(),
		LegFromCoords:    legTransferFrom.LegToCoords,
		LegToCoords:      legTransferTo.LegFromCoords,
		LegBeginTime:     legTransferFrom.LegEndTime,
		LegEndTime:       legTransferTo.LegBeginTime,
		TransferFrom:     legTransferFrom.TripTo,
		TransferTo:       legTransferTo.TripFrom,
		TransferDdwell:   ptr(transferDurationDwell.String()),
		TransferDactive:  ptr(transferDurationActive.String()),
		TransferFeedIDs:  ptr([]int{useTransfer.FromFeedID, useTransfer.ToFeedID}),
		TransferCategory: ptr(useTransfer.TransferCategory),
	}
}

func transferLookup(transfers *[]dbt.Transfer, fromSIDU, toSIDU string) *dbt.Transfer {
	for i, t := range *transfers {
		if t.FromSSIDU == fromSIDU && t.ToSSIDU == toSIDU {
			return &(*transfers)[i]
		} else if t.FromSSIDU == toSIDU && t.ToSSIDU == fromSIDU {
			return &(*transfers)[i]
		}
	}
	return nil
}
