package apirtroute

import (
	"time"

	"git.sr.ht/~mil/mobroute/api/apirtroute/diagnosticst"
	"git.sr.ht/~mil/mobroute/api/apirtroute/formattergeojson"
	"git.sr.ht/~mil/mobroute/api/apirtroute/formatterlegs"
	"git.sr.ht/~mil/mobroute/dbt"
)

// RouteParams contains the parameters for a given route request.
//
// The FeedIDs, From, To, TransferCategories, and Output format fields are
// required.
//
// Other fields are optional and will be filled in with default values
// if unset
type RouteParams struct {
	// Required
	FeedIDs            []int      `json:"feed_ids" validate:"required,gt=0"`
	From               [2]float64 `json:"from" validate:"required,len=2"`
	To                 [2]float64 `json:"to" validate:"required,len=2"`
	TransferCategories []string   `json:"transfer_categories" validate:"required,dive,oneof=f i g"`
	OutputFormats      []string   `json:"output_formats" validate:"required,gt=0,dive,oneof=connections legs geojson mapurl diagnostics request"`

	// Non-Required (has default values - see RTRoute logic)
	Time               *time.Time `json:"time" validate:"required"`                                     // Default: now
	MaxNTransfers      uint       `json:"max_n_transfers" validate:"required"`                          // Default: 20
	MaxTransferSeconds uint       `json:"max_transfer_seconds" validate:"gtfield=MinTransferSeconds"`   // Default: 40*60
	MaxTripSeconds     uint       `json:"max_trip_seconds" validate:"gt=0"`                             // Default: 240*60
	MaxWalkSeconds     uint       `json:"max_walk_seconds" validate:"gt=0"`                             // Default: 20*60
	MinTransferSeconds uint       `json:"min_transfer_seconds" validate:"required"`                     // Default: 3*60
	WalkspeedKmHr      float64    `json:"walkspeed_km_hr" validate:"gt=0"`                              // Default: 3.5
	Optimizations      []string   `json:"optimizations" validate:"required,gt=0,dive,oneof=feedsclean"` // Default: [feedsclean]
}

// RouteResponse is the response for a successful routing request.
type RouteResponse struct {
	RouteLegs        *[]formatterlegs.RouteLeg          `json:"legs,omitempty"`
	GeoJSON          *[]formattergeojson.GeoJSONFeature `json:"geojson,omitempty"`
	MapURL           *string                            `json:"mapurl,omitempty"`
	Request          *RouteParams                       `json:"request,omitempty"`
	RouteDiagnostics *diagnosticst.RouteDiagnostics     `json:"diagnostics,omitempty"`
	Connections      *[]dbt.ConnectionVerbose           `json:"connections,omitempty"`
}
