package apirtstops

import (
	"fmt"

	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/dbquery"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"github.com/go-playground/validator/v10"
)

func RTStops(runtime *apirtinit.MobrouteRuntime, params *StopsParams) (*StopsResponse, error) {
	utillog.Infoj("Running RTStops against input:", params)

	if err := validator.New(validator.WithRequiredStructEnabled()).Struct(params); err != nil {
		return nil, fmt.Errorf("Validation failed request: %v", err)
	} else if err := dbquery.QueryFeedIDsLoadedAndComputed(runtime.DB, params.FeedIDs); err != nil {
		return nil, err
	} else if stopWalks, err := dbquery.LoadStopWalks(
		runtime.DB,
		params.FeedIDs,
		params.FromCoord,
		params.MaxDistKM,
		params.Glob,
		utilfuncs.Fif(params.NLimit == 0, -1, params.NLimit),
	); err != nil {
		return nil, err
	} else {
		return &StopsResponse{
			Stops: stopWalks,
		}, nil
	}
}
