package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
)

var (
	subcmdDatabaseExamples = map[string]string{
		"Load a Mobility Database GTFS Feed":            `mobroute database -p '{"database_params": {"op": "load", "feed_ids": [767]}}'`,
		"Load a Custom GTFS Feed":                       `mobroute database -p '{"database_params": {"op": "loadcustomgtfs", "feed_ids": [-33], "loadcustomgtfs_uri": "http://web.mta.info/developers/data/nyct/subway/google_transit.zip"}}'`,
		"Compute Routing-Optimized GTFS Derived Tables": `mobroute database -p '{"database_params": {"op": "compute", "feed_ids": [767]}}'`,
		"Purge GTFS Feed Data":                          `mobroute database -p '{"database_params": {"op": "purge_gtfs", "feed_ids": [767]}}'`,
		"Purge Computed Tables":                         `mobroute database -p '{"database_params": {"op": "purge_computed", "feed_ids": [767]}}'`,
		"Purge All Tables":                              `mobroute database -p '{"database_params": {"op": "purge_all", "feed_ids": [767]}}'`,
	}
)

const (
	subcmdDatabaseDesc = `
The database subcommand provides diagnostic functionality allowing you to
alter the local SQLite database to import GTFS data (load), prepare routing
optimizated tables (compute), and remove existing data (purge). This is
essentially a passthrough to mobroute's underlying data ETL library mobsql.

After running database subcommands it's also helpful to query the database
using the databaseq command for the effected feed IDs.
`

	subcmdDatabaseDocParams = `
JSON Parameters for oneshot database subcommand, should be in format of: {"database_params": {"feed_ids": [767], "op": "load"}}

Required database parameters:
  - feed_ids: Array of integer feed IDs (MDBID) to use for database requests (ex: [767])
  - op: Operation to perform - may be one of "load", "loadcustomgtfs", "compute", "purge_gtfs", "purge_computed", "purge_all" (ex: "load")
`
)

func cmdDatabase(flagDatabaseSet *flag.FlagSet, flagDatabaseParams *string) {
	var databaseParams mobroute.OneshotDatabaseRequest
	flagDatabaseSet.Usage = usageFunc("database", flagDatabaseSet, subcmdDatabaseDesc, subcmdDatabaseExamples)
	flagDatabaseSet.Parse(os.Args[2:])
	if *flagDatabaseParams == "" {
		flagDatabaseSet.Usage()
	} else if err := json.Unmarshal([]byte(*flagDatabaseParams), &databaseParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseParams, err)
	} else if commandData, err := mobroute.OneshotDatabase(&databaseParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseParams, err)
	} else if jsonBytes, err := json.MarshalIndent(commandData, "", "  "); err != nil {
		errorDie(os.Args[1], *flagDatabaseParams, err)
	} else {
		fmt.Println(string(jsonBytes))
	}
}
