package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
)

var (
	subcmdDatabaseqExamples = map[string]string{
		"Determine the status of all GTFS feeds":   `mobroute databaseq -p '{"databaseq_params": {"feed_ids": []}}'`,
		"Determine the status of single GTFS feed": `mobroute databaseq -p '{"databaseq_params": {"feed_ids": [767]}}'`,
	}
)

const (
	subcmdDatabaseqDesc = `
The databaseq subcommand queries the underlying SQLite database to
determine the loaded/computed status of particular GTFS feed IDs. This
subcommand can be run after the database command (and before route,
schedule, stops) commands to ensure data is loaded/computed.
	`

	subcmdDatabaseqDocParams = `
JSON Parameters for the databaseq subcommand, should be in format of: {"databaseq_params": {"feed_ids": [767]}

Required database parameters:
  - feed_ids: Array of integer feed IDs (MDBID) to use for databaseq request (ex: [767])
`
)

func cmdDatabaseq(flagDatabaseqSet *flag.FlagSet, flagDatabaseqParams *string) {
	var databaseqParams mobroute.OneshotDatabaseqRequest
	flagDatabaseqSet.Usage = usageFunc("databaseq", flagDatabaseqSet, subcmdDatabaseqDesc, subcmdDatabaseqExamples)
	flagDatabaseqSet.Parse(os.Args[2:])
	if *flagDatabaseqParams == "" {
		flagDatabaseqSet.Usage()
	} else if err := json.Unmarshal([]byte(*flagDatabaseqParams), &databaseqParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseqParams, err)
	} else if commandData, err := mobroute.OneshotDatabaseq(&databaseqParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseqParams, err)
	} else if jsonBytes, err := json.MarshalIndent(commandData, "", "  "); err != nil {
		errorDie(os.Args[1], *flagDatabaseqParams, err)
	} else {
		fmt.Println(string(jsonBytes))
	}
}
