// Package main is the CLI interface for mobroute consuming the mobroute library.
// See mobroute -h for usage information.
package main

import (
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute/util/utilfuncs"
)

func main() {
	var (
		flagDatabaseSet    = flag.NewFlagSet("database", flag.ExitOnError)
		flagDatabaseParams = flagDatabaseSet.String("p", "", subcmdDatabaseDocParams)

		flagDatabaseqSet    = flag.NewFlagSet("databaseq", flag.ExitOnError)
		flagDatabaseqParams = flagDatabaseqSet.String("p", "", subcmdDatabaseDocParams)

		flagRouteSet    = flag.NewFlagSet("route", flag.ExitOnError)
		flagRouteParams = flagRouteSet.String("p", "", subcmdRouteDocParams)

		flagStopsSet    = flag.NewFlagSet("stops", flag.ExitOnError)
		flagStopsParams = flagStopsSet.String("p", "", subcmdStopsDocParams)

		flagScheduleSet    = flag.NewFlagSet("schedule", flag.ExitOnError)
		flagScheduleParams = flagScheduleSet.String("p", "", subcmdScheduleDocParams)
	)

	// Debug Same usage for all subcommands
	for _, f := range []*flag.FlagSet{
		flagDatabaseSet,
		flagDatabaseqSet,
		flagRouteSet,
		flagStopsSet,
		flagScheduleSet,
	} {
		var b bool
		f.BoolVar(&b, "h", false, "Show this usage message")
	}
	globalDeferedFn := globalsSetup()
	defer globalDeferedFn()

	// Run command
	validSubcommands := []string{"version", "route", "stops", "schedule", "database", "databaseq"}
	if len(os.Args) < 2 || !utilfuncs.StringInArray(validSubcommands, os.Args[1]) {
		fmt.Fprintf(os.Stderr, cmdDoc, os.Args[0])
		os.Exit(1)
	} else if os.Args[1] == "route" {
		cmdRoute(flagRouteSet, flagRouteParams)
	} else if os.Args[1] == "stops" {
		cmdStops(flagStopsSet, flagStopsParams)
	} else if os.Args[1] == "schedule" {
		cmdSchedule(flagScheduleSet, flagScheduleParams)
	} else if os.Args[1] == "database" {
		cmdDatabase(flagDatabaseSet, flagDatabaseParams)
	} else if os.Args[1] == "databaseq" {
		cmdDatabaseq(flagDatabaseqSet, flagDatabaseqParams)
	} else if os.Args[1] == "version" {
		fmt.Println("v0.8.4")
	}
}
