package dbquery

import (
	"fmt"
	"strings"

	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
)

// LoadAgencyTimezones simply extracts the timezone names for the given
// query feed ids.
func LoadAgencyTimezones(DB *sqlx.DB, queryFeedIDs []int) (*[]string, error) {
	type Timezones struct {
		TimezoneName string `db:"agency_timezone"`
	}
	var (
		r     = []Timezones{}
		query = fmt.Sprintf(
			"select agency_timezone from agency where feed_id in (%s)",
			strings.Join(utilfuncs.ArrayWithValue(len(queryFeedIDs), "?"), ","),
		)
		params = utilfuncs.Anyify(utilfuncs.ArrayIntToArryString(queryFeedIDs))
	)
	if err := queryToSlice(DB, query, params, &r); err != nil {
		return nil, err
	} else {
		arr := []string{}
		for _, t := range r {
			arr = append(arr, t.TimezoneName)
		}
		return &arr, nil
	}
}
