package dbquery

import (
	"fmt"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
	"strings"
)

// LoadStopWalks loads stopwalks given the input origin coordinate and a max
// distance in km to walk
func LoadStopWalks(
	DB *sqlx.DB,
	queryFeedIDs []int,
	originPoint [2]float64,
	maxWalkKM float64,
	glob string,
	nLimit int,
) (*[]dbt.StopWalk, error) {
	var (
		r     = []dbt.StopWalk{}
		query = fmt.Sprintf(
			query_stopwalks,
			strings.Join(utilfuncs.ArrayWithValue(len(queryFeedIDs), "?"), ","),
		)
		params = utilfuncs.Flatten([][]any{
			[]any{originPoint[0], originPoint[1], maxWalkKM},
			utilfuncs.Anyify(utilfuncs.ArrayIntToArryString(queryFeedIDs)),
			[]any{glob, glob, nLimit},
		})
	)
	if err := queryToSlice(DB, query, params, &r); err != nil {
		return nil, err
	} else {
		return &r, nil
	}
}
