package dbquery

import (
	"fmt"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
	"strings"
)

// LoadTransfers load the transfers from three seperate categories flagged
// by transferCategories:
//
// "f" - Feed-provided transfers contained in transfers.txt
// "i" - Implicit transfers (transfers from one stop to the same stop)
// "g" - Generated transfres (transfers from one stop to a different stop)
//
// Note - IntArraysSortedFilter logic makes sures pairs are not duplicate e.g.
// [[1 3] [3 1] [5 1]] is reduced to [[1 3] [5 1]]
func LoadTransfers(
	DB *sqlx.DB,
	queryFeedIDs []int,
	minTransferSeconds uint,
	maxTransferSeconds uint,
	walkspeedKmHr float64,
	transferCategories []string,
) (*[]dbt.Transfer, error) {
	var (
		r           = []dbt.Transfer{}
		feedIDPairs = utilfuncs.IntArraysSortedFilter(utilfuncs.Pairs(queryFeedIDs))
		query       = fmt.Sprintf(
			query_transfers,
			strings.Join(utilfuncs.ArrayWithValue(len(feedIDPairs), "(from_feed_id = ? and to_feed_id = ?)"), " or "),
			strings.Join(utilfuncs.ArrayWithValue(len(transferCategories), "?"), " , "),
		)
		params = utilfuncs.Flatten([][]any{
			[]any{
				// Implicit
				minTransferSeconds,

				// Generated
				walkspeedKmHr,
				minTransferSeconds,
				minTransferSeconds,

				// Feed
				walkspeedKmHr,
				minTransferSeconds,
				minTransferSeconds,
			},
			utilfuncs.Anyify(utilfuncs.Flatten(feedIDPairs)),
			[]any{any(maxTransferSeconds)},
			utilfuncs.Anyify(transferCategories),
		})
	)
	if err := queryToSlice(DB, query, params, &r); err != nil {
		return nil, err
	} else {
		return &r, nil
	}
}
