package dbquery

import (
	"fmt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
	"strings"
)

// QueryBoundsMatch is a querying function that determines if the passed
// param matches for the feed_id for the DB conneciton min/max lat/lon
func QueryBoundsMatch(
	DB *sqlx.DB,
	queryFeedIDs []int,
	coord [2]float64,
) ([]int, error) {
	var (
		boundsMatchFeedIDs []int
		query              = fmt.Sprintf(
			`
				select feed_id from (
					select 
						feed_id,
						(? between min_lat and max_lat) and
						(? between min_lon and max_lon) as matches_bounds
					from _vfeedsmetadata
					where feed_id in (%s) and matches_bounds
				) g
			`,
			strings.Join(utilfuncs.ArrayWithValue(len(queryFeedIDs), "?"), ","),
		)
		params = append(
			[]any{coord[0], coord[1]},
			utilfuncs.Anyify(utilfuncs.ArrayIntToArryString(queryFeedIDs))...,
		)
	)

	if err := DB.Select(&boundsMatchFeedIDs, query, params...); err != nil {
		return nil, fmt.Errorf("QueryBoundsMatch db select failure: %v", err)
	} else if len(boundsMatchFeedIDs) < 1 {
		return nil, fmt.Errorf("%v doesn't match any of the provided bounds for feed ids: %v", coord, queryFeedIDs)
	} else {
		return boundsMatchFeedIDs, nil
	}
}
