package dbschemaextra

import (
	_ "embed"
	"git.sr.ht/~mil/mobsql"
)

//go:embed sqlviews/_vstopparent.sql
var SQL_vstopparent string

//go:embed sqlviews/_vconn.sql
var SQL_vconn string

//go:embed sqlviews/_vconnverbose.sql
var SQL_vconnverbose string

//go:embed sqlviews/_vcaltoservice.sql
var SQL_vcaltoservice string

//go:embed sqlviews/_vtransferstbl.sql
var SQL_vtransferstbl string

//go:embed sqlviews/_vtransfersgen.sql
var SQL_vtransfersgen string

//go:embed sqlviews/_vtransfersgen.init.sql
var SQL_vtransfersgen_init string

//go:embed sqlviews/_vtransfersgen.deinit.sql
var SQL_vtransfersgen_deinit string

//go:embed sqlviews/_vtransfersimplicit.sql
var SQL_vtransfersimplicit string

//go:embed pragmas.sql
var pragmas string

var MobsqlSchemaExtra = mobsql.SchemaExtra{
	InitExec: pragmas,
	Views: []mobsql.View{
		{Name: "_vstopparent", Doc: "Mapping of stops to their parent stops", SQLCreateStatement: SQL_vstopparent},
		{Name: "_vconn", Doc: "CSA timetable", SQLCreateStatement: SQL_vconn},
		{Name: "_vconnverbose", Doc: "CSA timetable (with verbose joins)", SQLCreateStatement: SQL_vconnverbose},
		{Name: "_vcaltoservice", Doc: "Maps calendar dates individually to valid service id", SQLCreateStatement: SQL_vcaltoservice},
		{Name: "_vtransferstbl", Doc: "Transfers table added with from/to idu (parents)", SQLCreateStatement: SQL_vtransferstbl},
		{Name: "_vtransfersgen", Doc: "Transfers table generated from stops self-join", SQLCreateStatement: SQL_vtransfersgen},
		{Name: "_vtransfersimplicit ", Doc: "Implicit transfers ", SQLCreateStatement: SQL_vtransfersimplicit},
	},
	TablesComputed: []mobsql.ComputedTable{
		{

			SQLSelectView: `_vconn`,
			Doc:           `Core timetable`,
			CacheOnTables: []string{"stop_times", "trips", "stops", "routes"},
			TableSpec: mobsql.TableSpec{
				Table: "_ctconn",
				Indexes: [][]string{
					// Used for LoadConnections
					{"feed_id", "service_id", "from_stop_time"},
					// Used for LoadConnectionsVerbose
					{"feed_id", "from_stop_sidu", "from_stop_time"},
				},
				UniqueSet: []string{"feed_id", "conn_id"},
				Columns: []mobsql.LoadColumn{
					{Column: "feed_id", DType: mobsql.DTypeInt, Required: true},
					{Column: "conn_id", DType: mobsql.DTypeText, Required: true},
					{Column: "trip_id", DType: mobsql.DTypeText, Required: true},
					{Column: "service_id", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_time", DType: mobsql.DTypeInt, Required: true},
					{Column: "from_stop_sequence", DType: mobsql.DTypeInt, Required: true},
					{Column: "to_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_time", DType: mobsql.DTypeInt, Required: true},
					{Column: "to_stop_sequence", DType: mobsql.DTypeInt, Required: true},
				},
			},
		},
		{
			SQLSelectView: "_vcaltoservice",
			Doc:           "Maps each calendar day to its services",
			FeedPair:      false,
			CacheOnTables: []string{"stops"},
			TableSpec: mobsql.TableSpec{
				Table: "_ctcaltoservice",
				Indexes: [][]string{
					// Used for LoadConnectionVerbose (and optimizes schedules lib call)
					{"feed_id", "service_id", "service_date"},
					// Used for LoadConnections
					{"feed_id", "service_date"},
				},
				Columns: []mobsql.LoadColumn{
					{Column: "feed_id", DType: mobsql.DTypeInt, Required: true},
					{Column: "service_date", DType: mobsql.DTypeInt, Required: true},
					{Column: "service_id", DType: mobsql.DTypeText, Required: true},
				},
			},
		},
		{
			SQLSelectView: "_vtransfersgen",
			SQLInit:       SQL_vtransfersgen_init,
			SQLDeinit:     SQL_vtransfersgen_deinit,
			Doc:           "Generated transfers based on distances of stops",
			FeedPair:      true,
			CacheOnTables: []string{"stops"},
			TableSpec: mobsql.TableSpec{
				Table: "_cttransfersgen",
				Columns: []mobsql.LoadColumn{
					{Column: "feed_id", DType: mobsql.DTypeInt, Required: true},
					{Column: "feed_id_pair", DType: mobsql.DTypeInt, Required: true},
					{Column: "from_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "dist_km", DType: mobsql.DTypeReal},
				},
			},
		},
	},
}
