with recursive feedid_min_max_dates as (
  -- Min/mins both calendar/calendar_dates to get the range of valid dates
  select
    feed_id,
    min(start_date) as min_date,
    max(end_date) as max_date
  from (
    select feed_id, start_date, end_date from calendar
    union
    select feed_id, date as start_date, date as end_date from calendar_dates
  )
  group by feed_id
),
psmmd as (
  -- (Parsed feed_id min max dates) parses min max dates yyyymmdd to yyyy-mm-dd
  -- this is done as sqlite only understands yyyy-mm-dd for dates
  select
    feed_id,
    printf(
      '%s-%s-%s',
      substr(min_date, 1, 4), substr(min_date, 5, 2), substr(min_date, 7, 2)
    ) as min_date,
    printf(
      '%s-%s-%s',
      substr(max_date, 1, 4), substr(max_date, 5, 2), substr(max_date, 7, 2)
    ) as max_date
    from feedid_min_max_dates
),
date_range as (
  -- Expands date range for each feed_id in psmmd for each feed_id
  -- ex: range is 2021-02-02 to 2021-02-05 that would expand to 3 rows
  select psmmd.feed_id, psmmd.min_date as inputdate from psmmd
  union
  select date_range.feed_id, date(date_range.inputdate, '+1 day') as inputdate
  from date_range join psmmd on date_range.feed_id = psmmd.feed_id
  where date_range.inputdate < psmmd.max_date
),
feedid_to_dates_mapping as (
  -- (Date range component): extract each date's wkday component & yyyymmdd
  -- for simpler comparisons
  select
    feed_id,
    inputdate as dateiso8601,
    cast(strftime('%Y%m%d', inputdate) as int) as dateyyyymmdd,
    cast(strftime('%w', inputdate) as int) as datewd
  from date_range
)
select feed_id, dateyyyymmdd as service_date, service_id from (
  -- Calendar dates feed_id to service
  -- (include exception_type 1 only; explicitly exclude exception_type 2)
  select
    cd.feed_id, cd.service_id, v.dateyyyymmdd
  from feedid_to_dates_mapping v
  left join calendar_dates cd
    on cd.feed_id = v.feed_id and v.dateyyyymmdd = cd.date and cd.exception_type = 1
  left join calendar_dates cde
    on cde.feed_id = v.feed_id and v.dateyyyymmdd = cde.date and cd.service_id = cde.service_id and cde.exception_type = 2
  where cd.exception_type = 1 and cde.exception_type is null

  union all

  -- Calendar range feed_id to service
  -- (explicitly exclude exception_type 2 from calendar_dates)
  select
    c.feed_id, c.service_id, v.dateyyyymmdd
  from feedid_to_dates_mapping v
  left join calendar c
    on c.feed_id = v.feed_id and v.dateyyyymmdd between c.start_date and c.end_date and (
      (c.sunday and v.datewd = 0)
      or (c.monday and v.datewd = 1)
      or (c.tuesday and v.datewd = 2)
      or (c.wednesday and v.datewd = 3)
      or (c.thursday and v.datewd = 4)
      or (c.friday and v.datewd = 5)
      or (c.saturday and v.datewd = 6)
    )
  left join calendar_dates cde
    on cde.feed_id = v.feed_id and v.dateyyyymmdd = cde.date and c.service_id = cde.service_id and cde.exception_type = 2
  where c.service_id is not null and cde.exception_type is null
) g
 -- Optimization to cap between 2024-2028, will need revisiting at somepoint
where dateyyyymmdd between 20230000 and 20280000