with
rough_bbox_intersects as (
  select
    s1.feed_id as s1_feed_id,
    s1.stop_id as s1_stop_id,
    s1.stop_lat as s1_stop_lat,
    s1.stop_lon as s1_stop_lon,

    s2.feed_id as s2_feed_id,
    s2.stop_id as s2_stop_id,
    s2.stop_lat as s2_stop_lat,
    s2.stop_lon as s2_stop_lon

  from stops_ridx s1
  join stops_ridx s2
    on (s1.stop_lat between s2.lat_min and s2.lat_max)
    and (s1.stop_lon between s2.lon_min and s2.lon_max)
    and (s1.feed_id < s2.feed_id or (s1.feed_id = s2.feed_id and s2.id < s1.id))
),
precise_bbox_intersects as (
  select
    *,
    6371.0 * acos(
      sin(90.0 - rbi.s2_stop_lat * pi() / 180.0) *
      sin(90.0 - rbi.s1_stop_lat * pi() / 180.0) *
      cos(
        (rbi.s2_stop_lon * pi() / 180.0) -
        (rbi.s1_stop_lon * pi() / 180.0)
      ) +
      cos(90.0 - rbi.s2_stop_lat * pi() / 180.0) *
      cos(90.0 - rbi.s1_stop_lat * pi() / 180.0)
    ) as dist_km
  from rough_bbox_intersects rbi
  where dist_km < 3.0
)
select
  cast(pbi.s1_feed_id as int) as feed_id,
  cast(pbi.s2_feed_id as int) as feed_id_pair,
  pbi.s1_stop_id as from_stop_id,
  pbi.s2_stop_id as to_stop_id,
  pbi.dist_km,
  pbi.s1_stop_id as from_stop_idu, -- Note filtered stop_id = stop_idu (parent) at rtable currently
  pbi.s2_stop_id as to_stop_idu, -- Note filtered stop_id = stop_idu (parent) at rtable currently
  cast(pbi.s1_feed_id as int)|| '_' || pbi.s1_stop_id as from_stop_sidu,
  cast(pbi.s2_feed_id as int)|| '_' || pbi.s2_stop_id as to_stop_sidu
from precise_bbox_intersects pbi
