package dbstubquery

import (
	"git.sr.ht/~mil/mobroute/dbt"
	"github.com/jmoiron/sqlx"
)

// LoadCalToService loads the mapping of calendar days to valid services
// for that day (modeled by the underlying _vcaltoservice view).
//
// This code is not directly used by the routing library, but the underlying
// _vcaltoservice view is used as a CTE in LoadConnections (and in combination
// with its _ctconn table) ultimately to determine which connections get
// loaded for a routing request time duration.

// As such, the existance of this function is to have unit-tests for the
// fucntionality of determining *which* services are valid for a given input
// date. And to have the ability to to individually / in isolation test
// the functionality of the _vcaltoservice view which is responsible for this.
func LoadCalToService(
	DB *sqlx.DB,
	feedID, fromDateYYYYMMDD, toDateYYYYMMDD uint,
) (*[]dbt.CalToService, error) {
	var (
		r     = []dbt.CalToService{}
		query = `
			select feed_id, service_date, service_id from _ctcaltoservice
			where feed_id = ? and service_date between ? and ?
		`
		params = []any{any(feedID), any(fromDateYYYYMMDD), any(toDateYYYYMMDD)}
	)
	if err := queryToSlice(DB, query, params, &r); err != nil {
		return nil, err
	} else {
		return &r, nil
	}
}
