package dbstubquery

import (
	"git.sr.ht/~mil/mobroute/dbt"
	"github.com/jmoiron/sqlx"
)

// LoadStopParent loads the mapping of the stop to its 'ultimate' parent
// based on the underlying _vstopparent view. Given that GTFS's stops.txt
// provides a 'hierarchical' structure based on the parent_station, the
// _vstopparent view 'flattens' this and provides a mapping for each
// stop to its "ultimate" parent.
//
// This loader is only used in tests, and essentially functions to provide
// a functional test for the view logic in _vstopparent
func LoadStopToParent(DB *sqlx.DB, feedID uint) (*[]dbt.StopToParent, error) {
	var (
		r     = []dbt.StopToParent{}
		query = `
			select feed_id, stop_id, parent_station from _vstopparent
			where feed_id = ?
		`
		params = []any{any(feedID)}
	)
	if err := queryToSlice(DB, query, params, &r); err != nil {
		return nil, err
	} else {
		return &r, nil
	}
}
