#!/usr/bin/env sh
cd "$(realpath "$(dirname "$0")")"/.. || exit 1

# E.g. used to generate CLI doc:
# ./generate_cliguide.sh > doc/userguide_cli.md

HEADER="$(cat <<EOF
# **Mobroute:** CLI Userguide

[Mobroute](http://sr.ht/~mil/mobroute) is a general purpose FOSS public
transportation router (e.g. trip planner) Go library and CLI that works by
directly ingesting timetable (GTFS) data from transit agencies themselves
(sourced from the Mobility Database). For an overview of Mobroute's
functionality see [Mobroute's project page](http://sr.ht/~mil/mobroute).
This document is a guide on how to run Mobroute's CLI by compiling the
Go codebase and then running routing requests locally.

- *1:* [Primer Documentation](#strongprimer-documentationstrong)
- *2:* [Mobroute CLI: Install](#strongmobroute-cli-installstrong)
- *3:* [Mobroute CLI: usage](#strongmobroute-cli-usagestrong)
- *4:* [Mobroute CLI: route](#strongmobroute-cli-routestrong)
- *5:* [Mobroute CLI: stops](#strongmobroute-cli-stopsstrong)
- *6:* [Mobroute CLI: schedule](#strongmobroute-cli-schedulestrong)
- *7:* [Mobroute CLI: database](#strongmobroute-cli-databasestrong)
- *8:* [Mobroute CLI: databaseq](#strongmobroute-cli-databaseqstrong)
- *9:* [Further Resources & Documentation](#strongfurther-resources-amp-documentationstrong)

## **Primer Documentation**

- Before using Mobroute's CLI, please read the following documentation:
  - [Mobroute: Project Overview & Featureset](https://git.sr.ht/~mil/mobroute)
  - [Mobroute: Finding GTFS Feeds](https://git.sr.ht/~mil/mobroute/tree/master/doc/doc_gtfsfeeds.md)
  - [Mobroute: Algorithm Tunable Parameters](https://git.sr.ht/~mil/mobroute/tree/master/doc/doc_tunables.md)

## **Mobroute CLI: Install**

- Install via package:
  - On some distrobutions the Mobroute CLI has been packaged, if available
    this is the simplest installation method.
  - Distrobution packages:
    - Alpine Linux Testing: \`apk add mobroute\`


- Install from source:
  - If a package is not avilabale for your distrobution, building from source
    is simple using Go
  - Steps:
    - (1) Installation assumes Go (>=1.18)
    - (2) Clone the Mobroute repository:
      - \`git clone http://git.sr.ht/~mil/mobroute\`
    - (3) Build the \`mobroute\` binary:
      - \`./build.sh\`
    - (4) Install as:
      - \`cp mobroute /usr/bin/\`


EOF
)"

FOOTER="$(cat <<EOF

## **Further Resources & Documentation**

- [Automated Test Results / Metros](https://ci.lrdu.org/tests_results/): If you're curious about metros that have been well-tested and MDBIDs which you can experiment with for routing, this page has a number of working routing requests (lat/lon pairs and MDBIDs)
- [Development Guide](doc_dev.md): If you're curious about how Mobroute works under-the-hood, this guide can help architecture overview of what happens when you submit a routing request, from ETL, to algorithm, to formatting
- [Contribute Guide](doc_contribute.md): If you're intrested in contributing, this goes over the workflow.
- [Transito](http://git.sr.ht/~mil/transito): The Transito Mobile app runs the same routing logic the CLI executes on a mobile app (Android & Linux)

---

> [All Mobroute Docs](http://sr.ht/~mil/mobroute)

EOF
)"

gendoc() {
  SUBCMD=$1
  ./build.sh run $SUBCMD 2>&1
}

mddoc() {
  echo "$HEADER"

  echo '## **Mobroute CLI: usage**'
  echo '```'
  gendoc usage
  echo '```'
  echo ''

  echo '## **Mobroute CLI: route**'
  echo '```'
  gendoc route
  echo '```'
  echo ''

  echo '## **Mobroute CLI: stops**'
  echo '```'
  gendoc stops
  echo '```'
  echo ''

  echo '## **Mobroute CLI: schedule**'
  echo '```'
  gendoc schedule
  echo '```'
  echo ''

  echo '## **Mobroute CLI: database**'
  echo '```'
  gendoc database
  echo '```'
  echo ''

  echo '## **Mobroute CLI: databaseq**'
  echo '```'
  gendoc databaseq
  echo '```'
  echo ''

  echo "$FOOTER"
}

mddoc
