package testhelper

import (
	"encoding/csv"
	"fmt"
	"os"
	"sort"
)

// ArrayOfMapsToCSV converts an array of maps into a CSV file
func ArrayOfMapsToCSV(arr []map[string]any, targetCSVFile string) error {
	var (
		csvFile *os.File
		err     error
	)

	if csvFile, err = os.Create(targetCSVFile); err != nil {
		return err
	}
	defer csvFile.Close()
	writer := csv.NewWriter(csvFile)
	defer writer.Flush()
	csvCols := allKeys(arr)
	writer.Write(csvCols)
	for _, record := range arr {
		row := []string{}
		for _, key := range csvCols {
			if record[key] == nil {
				row = append(row, "")
			} else {
				row = append(row, fmt.Sprintf("%v", record[key]))
			}
		}
		writer.Write(row)
	}
	return nil
}

func allKeys(maps []map[string]any) []string {
	keysMap := map[string]interface{}{}
	for _, m := range maps {
		for k, _ := range m {
			keysMap[k] = nil
		}
	}
	keys := []string{}
	for k, _ := range keysMap {
		keys = append(keys, k)
	}
	sort.Strings(keys)
	return keys
}
