package dbquerytesthelper

import (
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
	"git.sr.ht/~mil/mobroute/testhelper"
	"github.com/jmoiron/sqlx"
)

func PrepareMockLoadComputeAndOpenDB(
	mapFeedIDToYML map[int]string,
) (*sqlx.DB, string, error) {
	feedIDs := []int{}
	for id, _ := range mapFeedIDToYML {
		feedIDs = append(feedIDs, id)
	}

	if mockMobilityDBCSV, gtfsZips, err := testhelper.PrepareMockMobilityDBMockFromGTFSYMLs(mapFeedIDToYML); err != nil {
		return nil, "", fmt.Errorf("Failed to prepare mock: %v", err)
	} else if sqliteDB, err := os.CreateTemp("", "mobroute_test_sqlite_*.db"); err != nil {
		return nil, "", fmt.Errorf("Failed to create mock SQLite DB: %v", err)
	} else if mrRuntime, err := mobroute.RTInitialize(
		&mobroute.MobrouteRuntimeConfig{
			MobsqlRuntimeConfig: mobroute.MobsqlRuntimeConfig{
				MobilityDBCatalogCSVURI: fmt.Sprintf("file://%s", mockMobilityDBCSV),
				SQLiteDBPath:            sqliteDB.Name(),
				LogInfo:                 true,
				LogWarn:                 true,
				LogDebug:                true,
			},
		},
	); err != nil {
		return nil, "", fmt.Errorf("Initialize runtime fail: %v: DB: %s", err, sqliteDB.Name())
	} else if _, err := mobroute.RTDatabase(mrRuntime, &mobroute.DatabaseParams{Op: "load", FeedIDs: feedIDs}); err != nil {
		return mrRuntime.DB, "", fmt.Errorf("PrepareMockLoadComputeAndOpenDB Load fail: %v, DB: %s", err, sqliteDB.Name())
	} else if _, err := mobroute.RTDatabase(mrRuntime, &mobroute.DatabaseParams{Op: "compute", FeedIDs: feedIDs}); err != nil {
		return mrRuntime.DB, "", fmt.Errorf("PrepareMockLoadComputeAndOpenDB Compute fail: %v, DB: %s", err, sqliteDB.Name())
	} else {
		os.Remove(mockMobilityDBCSV)
		for _, z := range gtfsZips {
			os.Remove(z)
		}
		return mrRuntime.DB, sqliteDB.Name(), nil
	}
}
