package utilfuncs

import (
	"fmt"
	"sort"
	"strconv"
)

func MapKeys[TK comparable, TV any](m map[TK]TV) []TK {
	keys := []TK{}
	for k, _ := range m {
		keys = append(keys, k)
	}
	return keys
}

func FilterMapByKeys[TK comparable, TV any](m map[TK]TV, validKeys []TK) map[TK]TV {
	returnMap := map[TK]TV{}
	for _, k := range validKeys {
		if v, exists := m[k]; exists {
			returnMap[k] = v
		}
	}
	return returnMap
}

func MaxInt(a, b uint) uint {
	if a > b {
		return a
	} else {
		return b
	}
}

func ArrayWithValue(size int, s string) []string {
	r := make([]string, size)
	for i := range r {
		r[i] = s
	}
	return r
}

func ArrayIntToArryString(strings []int) []string {
	r := []string{}
	for _, i := range strings {
		r = append(r, fmt.Sprintf("%d", i))
	}
	return r
}

func Anyify[T any](items []T) []any {
	r := []any{}
	for _, s := range items {
		r = append(r, any(s))
	}
	return r
}

func MapValueAdd(m map[string]uint, value int) map[string]int {
	r := map[string]int{}
	for k, v := range m {
		r[k] = int(v) + value
	}
	return r
}

func StringInArray(items []string, findItem string) bool {
	for _, item := range items {
		if item == findItem {
			return true
		}
	}
	return false
}

func KeyValues(m map[string]int) []any {
	r := []any{}
	for k, v := range m {
		r = append(r, any(k), any(v))
	}
	return r
}

func Flatten[T any](arrArrs [][]T) []T {
	r := []T{}
	for _, arr := range arrArrs {
		for _, a := range arr {
			r = append(r, a)
		}
	}
	return r
}

func Pairs[T any](items []T) [][]T {
	pairs := [][]T{}
	for _, s1 := range items {
		for _, s2 := range items {
			pairs = append(pairs, []T{s1, s2})
		}
	}
	return pairs
}

func IntArraysSortedFilter(intArrays [][]int) [][]int {
	r := [][]int{}
OUTER:
	for _, items := range intArrays {
		last := -99999
		for _, item := range items {
			if item < last {
				continue OUTER
			} else {
				last = item
			}
		}
		r = append(r, items)
	}
	return r
}

func IntArrayToStringArray(arrInt []int) []string {
	r := []string{}
	for _, item := range arrInt {
		r = append(r, strconv.Itoa(item))
	}
	return r
}

func ItemInArray[T comparable](item T, items []T) bool {
	for _, s := range items {
		if item == s {
			return true
		}
	}
	return false
}

func Fif[T any](cond bool, trueVal, falseVal T) T {
	if cond {
		return trueVal
	} else {
		return falseVal
	}
}

func UniqueInts(items []int) []int {
	var (
		keys = map[int]any{}
		arr  = []int{}
	)

	for _, item := range items {
		keys[item] = 0
	}
	for k, _ := range keys {
		arr = append(arr, k)
	}
	sort.Ints(arr)
	return arr
}
