package utilfuncs

import (
	"fmt"
	"time"
)

func DurationFormatSecs(seconds uint) string {
	if seconds < 60 {
		return fmt.Sprintf("%ds", seconds)
	} else {
		minutes := seconds / 60
		secsrem := seconds % 60
		return fmt.Sprintf("%dm %ds", minutes, secsrem)
	}
}

// TODO: is this better to do via time package (AM/PM formatting free then?)
func TimeHMSFormatSecs(seconds uint) string {
	var (
		s = seconds % 60
		m = (seconds / 60) % 60
		h = seconds / 3600
	)

	if s == 0 {
		return fmt.Sprintf("%02d:%02d", h, m)
	} else {
		return fmt.Sprintf("%02d:%02d:%02d", h, m, s)
	}
}

func DayUTCOffsetSecs(t *time.Time) int {
	_, utcOffsetSeconds := t.Zone()
	return utcOffsetSeconds
}

func DayYYYYMMDD(t *time.Time) uint {
	y, m, d := t.Date()
	return uint(y*10000) + uint((m * 100)) + uint(d)
}

func DaySeconds(t *time.Time) uint {
	y, m, d := t.Date()
	midnight := time.Date(y, m, d, 0, 0, 0, 0, t.Location())
	return uint(t.Sub(midnight).Seconds())
}

func TimezoneToUTCDeltaSecs(yyyymmdd uint, timezone string) (int, error) {
	var (
		year  = int(yyyymmdd / 10000)
		month = int((yyyymmdd / 100) % 100)
		day   = int(yyyymmdd % 100)
	)
	if location, err := time.LoadLocation(timezone); err != nil {
		return 0, err
	} else {
		t := time.Date(year, time.Month(month), day, 0, 0, 0, 0, location)
		_, offset := t.Zone()
		return offset, nil
	}
}

func TimezoneToUTCDeltaMap(timezones []string, yyyymmdd uint) (map[string]int, error) {
	m := map[string]int{}
	for _, tz := range timezones {
		if delta, err := TimezoneToUTCDeltaSecs(yyyymmdd, tz); err != nil {
			return nil, err
		} else {
			m[tz] = delta
		}
	}
	return m, nil
}

func CreateTimeFromYYYYMMDDZoneSecs(yyyymmdd uint, timezoneUTCSeconds, daySeconds int) time.Time {
	var (
		year     = int(yyyymmdd / 10000)
		month    = time.Month((yyyymmdd / 100) % 100)
		day      = int(yyyymmdd % 100)
		zone     = time.FixedZone("user_zone", timezoneUTCSeconds)
		midnight = time.Date(year, month, day, 0, 0, 0, 0, zone)
	)
	return midnight.Add(time.Duration(daySeconds-timezoneUTCSeconds) * time.Second)
}
