package utilfuncs_test

import (
	"fmt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"math"
	"testing"
)

func Test_DistHaversine(t *testing.T) {
	testCases := []struct {
		coordA [2]float64
		coordB [2]float64
		expect float64
	}{
		{coordA: [2]float64{2, 2}, coordB: [2]float64{2, 2}, expect: 0},
		{coordA: [2]float64{40.73586, -74.02922}, coordB: [2]float64{40.73586, -74.02922}, expect: 0},
		{coordA: [2]float64{20.2, 30.8883}, coordB: [2]float64{21.3, 30.87}, expect: 122.329220},
		{coordA: [2]float64{20.200, 30.8800}, coordB: [2]float64{20.2001, 30.88003}, expect: 0.011552},
		{coordA: [2]float64{1, 2}, coordB: [2]float64{2, 2}, expect: 111.19},
	}

	for _, testCase := range testCases {
		if err := floatsEqual(testCase.expect, utilfuncs.DistHaversine(testCase.coordA, testCase.coordB)); err != nil {
			t.Fatalf("%v", err)
		}
	}
}

func floatsEqual(a, b float64) error {
	threshold := 0.005
	if diff := math.Abs(a - b); diff < threshold {
		return nil
	} else {
		return fmt.Errorf("Comparison of a=%f b=%f exceeds threshold=%f", a, b, threshold)
	}
}
