package utilfuncs_test

import (
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"testing"
)

func Test_TimezoneToUTCDeltaSecs(t *testing.T) {
	for _, testCase := range []struct {
		TZ             string
		Date           uint
		ExpectUTCDelta int
		ExpectError    bool
	}{
		{TZ: "Europe/Berlin", Date: 20241027, ExpectUTCDelta: 7200},
		{TZ: "Europe/Berlin", Date: 20241028, ExpectUTCDelta: 3600},

		{TZ: "America/Chicago", Date: 20241103, ExpectUTCDelta: -18000},
		{TZ: "America/Chicago", Date: 20241104, ExpectUTCDelta: -21600},

		{TZ: "UTC", Date: 20241104, ExpectUTCDelta: 0},

		{TZ: "invalid tz", Date: 20241104, ExpectError: true},
	} {
		if utcdelta, err := utilfuncs.TimezoneToUTCDeltaSecs(testCase.Date, testCase.TZ); (err != nil) != testCase.ExpectError {
			t.Fatalf(
				"Expected errorstate %v; got error %v (for TZ=%v / Date=%v)",
				testCase.ExpectError, err, testCase.TZ, testCase.Date,
			)
		} else if testCase.ExpectUTCDelta != utcdelta {
			t.Fatalf(
				"Expected %v; got %v (for TZ=%v / Date=%v)",
				testCase.ExpectUTCDelta, utcdelta, testCase.TZ, testCase.Date,
			)
		}
	}
}
